@@ -1,4 +1,4 @@
-from typing import Any, Dict, Union, List
+from typing import Any, Dict, List, Union
 
 from cognite.client import utils
 from cognite.client.data_classes._base import CogniteResource, CogniteResourceList
@@ -10,6 +10,7 @@ class FeatureType(CogniteResource):
     def __init__(
         self,
         external_id: str = None,
+        data_set_id: int = None,
         created_time: int = None,
         last_updated_time: int = None,
         properties: Dict[str, Any] = None,
@@ -17,6 +18,7 @@ class FeatureType(CogniteResource):
         cognite_client=None,
     ):
         self.external_id = external_id
+        self.data_set_id = data_set_id
         self.created_time = created_time
         self.last_updated_time = last_updated_time
         self.properties = properties
