@@ -1,5 +1,6 @@
 from cognite.client.data_classes._base import *
 from cognite.client.data_classes.shared import TimestampRange
+from cognite.client.data_classes.transformations._alphatypes import AlphaDataModelInstances
 from cognite.client.data_classes.transformations.common import *
 from cognite.client.data_classes.transformations.jobs import TransformationJob, TransformationJobList
 from cognite.client.data_classes.transformations.schedules import TransformationSchedule
@@ -118,6 +119,12 @@ class Transformation(CogniteResource):
             if instance.destination.get("type") == "raw":
                 snake_dict.pop("type")
                 instance.destination = RawTable(**snake_dict)
+            elif instance.destination.get("type") == "data_model_instances":
+                snake_dict.pop("type")
+                instance.destination = AlphaDataModelInstances(**snake_dict)
+            elif instance.destination.get("type") == "sequence_rows":
+                snake_dict.pop("type")
+                instance.destination = SequenceRows(**snake_dict)
             else:
                 instance.destination = TransformationDestination(**snake_dict)
         if isinstance(instance.running_job, Dict):
@@ -147,14 +154,14 @@ class Transformation(CogniteResource):
             Dict[str, Any]: A dictionary representation of the instance.
         """
         ret = super().dump(camel_case=camel_case)
-
         if self.source_oidc_credentials:
             source_key = "sourceOidcCredentials" if camel_case else "source_oidc_credentials"
             ret[source_key] = self.source_oidc_credentials.dump(camel_case=camel_case)
         if self.destination_oidc_credentials:
             destination_key = "destinationOidcCredentials" if camel_case else "destination_oidc_credentials"
             ret[destination_key] = self.destination_oidc_credentials.dump(camel_case=camel_case)
-
+        if isinstance(self.destination, AlphaDataModelInstances) or isinstance(self.destination, SequenceRows):
+            ret["destination"] = self.destination.dump(camel_case=camel_case)
         return ret
 
     def __hash__(self):
@@ -221,6 +228,13 @@ class TransformationUpdate(CogniteUpdate):
     def data_set_id(self):
         return TransformationUpdate._PrimitiveTransformationUpdate(self, "dataSetId")
 
+    def dump(self, camel_case: bool = True):
+        obj = super().dump()
+        dest = obj.get("update", {}).get("destination", {}).get("set")
+        if isinstance(dest, AlphaDataModelInstances) or isinstance(dest, SequenceRows):
+            obj["update"]["destination"]["set"] = dest.dump(camel_case=camel_case)
+        return obj
+
 
 class TransformationList(CogniteResourceList):
     _RESOURCE = Transformation
